#include "enemy.h"
#include "bullet.h"
#include "player.h"

// static variable used to control general direction of enemy
// 1 means move right, -1 means move left (altered in main)
GLint enemy::direction = 1;
// constructor 
enemy::enemy(GLfloat _x, GLfloat _y, GLfloat _z)
{
	alive = true;

	mybullet = NULL;

	maxspeed[x] = 0.1; maxspeed[y] = -0.5; maxspeed[z] = 0.0;

	pos[x] = _x; pos[y] = _y; pos[z] = _z;

	// specify specular and diffuse color for top & bottom of spaceship
	bottom_dif[x] = 0.0; bottom_dif[y] = 0.5; bottom_dif[z] = 0.0;
	bottom_spc[x] = 0.1; bottom_spc[y] = 0.1; bottom_spc[z] = 0.1;

	top_dif[x] = 0.8; top_dif[y] = 0.8; top_dif[z] = 0.8;
	top_spc[x] = 0.5; top_spc[y] = 0.5; top_spc[z] = 0.5;

}

void enemy::draw()
{
	glPushMatrix();
		// draw bottom of spaceship
		glTranslatef(pos[x], pos[y], pos[z]);
		glPushMatrix();
			glMateriali(GL_FRONT, GL_SHININESS, 0);
			glMaterialfv(GL_FRONT, GL_DIFFUSE, bottom_dif);
			glMaterialfv(GL_FRONT, GL_SPECULAR, bottom_spc);
			glScalef(1.0, 0.25, 1.0);
			glutSolidSphere(1.0, 16, 16);
		glPopMatrix();
		
		// draw top of spaceship 
		glPushMatrix();
			glMateriali(GL_FRONT, GL_SHININESS, 96);
			glTranslatef(0.0, 0.25, 0.0);
			glMaterialfv(GL_FRONT, GL_DIFFUSE, top_dif);
			glMaterialfv(GL_FRONT, GL_SPECULAR, top_spc);
			glScalef(1.0, 0.75, 1.0);
			glutSolidSphere(0.5, 16, 16);
		glPopMatrix();
	glPopMatrix();
}

// calculate angle for firing
void enemy::getAngle(void *pplayer)
{

	GLfloat _x = pos[x] - ((player*)pplayer)->pos[x];
	GLfloat _y = pos[y] - ((player*)pplayer)->pos[y];

	fire_angle = atan(_x/_y) * 180 / 3.1415926;
}

void enemy::fire()
{
	deleteBullet();									// only allows 1 bullet at a time
	bullet *b = new bullet((void*)this);			// creates new bullet and sets the
	mybullet = b;									// player's void pointer to it

	((bullet*)mybullet)->speed[y] = -1 * ((bullet*)mybullet)->speed[y];
}

void enemy::deleteBullet()
{
	if (mybullet != NULL)
	{
		((bullet*)mybullet)->~bullet();
		delete mybullet;
		mybullet = NULL;
	}
}