/*
CMPS 473
Homework 6
Nick Toothman
5/15/09

This is a Space Invaders-type game.  The object of the game is to eliminate all of the enemy
spaceships before they reach the ground (same vertical level as you). As you eliminate more 
ships, the remaining ones begin to move faster.  Press A and D to move left and right, Left
and Right Arrows to aim your cannon, and spacebar to fire.  ESC to exit at any time.  

This application was written using Visual C++ in Visual Studio 2005.  It has been tested on
Visual C++ 6.0 and all of the functionality works as expected.  If there are any problems
compiling or testing, please contact me at wangninja@gmail.com or ntoothma@cs.csubak.edu
  */

#include "main.h"
#include "enemy.h"
#include "bullet.h"
#include "player.h"

GLint activeships = MAXSHIPS;

const GLint XLEFT = -20;
const GLint XRIGHT = 20;
const GLint YMAX = 30;
const GLfloat MAXANGLE = 60.0;
const GLfloat MINANGLE = -60.0;

// create player object and enemy array
player pplayer;
enemy *enemies[MAXSHIPS];

GLint playerscore = 0;
char *score = new char[30];
GLfloat textColor[3] = {1.0, 1.0, 1.0};

bool gamefinished = false;

void display()
{
	glClearColor(0.0, 0.0, 0.0, 0.0); // RGBA
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// camera by default looks in the -z direction
	glPushMatrix(); // put current matrix into stack
		glTranslatef(0.0, -5.0, -30.0); // x, y, z. moves object 30 units back
		glRotatef(angleViewX, 1.0, 0.0, 0.0);
		glRotatef(angleViewY, 0.0, 0.0, 1.0);   
		glPushMatrix();
			// only update score while game isn't finished
			if (!gamefinished) _itoa_s(playerscore, score, 10, 10);

			// print off score/endgame message
			textPrintf(-2.0, -1.0, GLUT_BITMAP_TIMES_ROMAN_24, score, textColor);
		glPopMatrix();

		// draw all ships that are alive
		for(int i = 0; i < MAXSHIPS; i++)
		{
			if (enemies[i]->alive)
			{
				enemies[i]->draw();
				if (enemies[i]->mybullet != NULL)
				{
					((bullet*)enemies[i]->mybullet)->draw();
				}
			}
		}

		// draw player
		pplayer.draw();

		// if the player has a bullet, draw it
		if (pplayer.mybullet != NULL)
		{
			((bullet*)pplayer.mybullet)->draw();
		}
	glPopMatrix(); // restore stack;
	glutSwapBuffers(); // GL uses two buffers to reduce flickering
	glutPostRedisplay();
}

// used to roughly compute distance between bullet and ship
GLfloat computeDistance(GLfloat pos1[3], GLfloat pos2[3])
{
	GLfloat _x = (GLfloat)pow((pos2[x] - pos1[x]), 2);
	GLfloat _y = (GLfloat)pow((pos2[y] - pos1[y]), 2);
	GLfloat _z = (GLfloat)pow((pos2[z] - pos1[z]), 2);

	return sqrt(_x + _y + _z);
}

// called when an endgame has been reached
void gameOver(bool playerWon)
{
	gamefinished = true;

	// player won
	if (playerWon)
	{
		score = "You won! Press ESC to exit";
		
	}

	else
	{
		score = "You lost. Press ESC to exit";
	}

}

void idle()
{
	if (!gamefinished)
	{
		// check for endgame activity
		if (activeships == 0 && pplayer.alive)
			gameOver(true);

		// update enemies' positions and check for survivors
		GLint lowest = MAXSHIPS, highest = 0;
		for(int i = 0; i < MAXSHIPS; i++)
		{
			if (enemies[i]->alive)
			{
				enemies[i]->pos[x] += (enemy::direction * enemies[i]->maxspeed[x]);
				enemies[i]->getAngle((void*)&pplayer);

				if (enemies[i]->mybullet != NULL)
				{
					((bullet*)enemies[i]->mybullet)->position[x] += ((bullet*)enemies[i]->mybullet)->speed[x];
					((bullet*)enemies[i]->mybullet)->position[y] += ((bullet*)enemies[i]->mybullet)->speed[y];
					((bullet*)enemies[i]->mybullet)->position[z] += ((bullet*)enemies[i]->mybullet)->speed[z];
				}
			}

			if (enemies[i]->alive && i < lowest) lowest = i;
			if (enemies[i]->alive && i > highest) highest = i;
		}

		// control motion of ships
		if (enemies[lowest]->pos[x] < XLEFT || enemies[highest]->pos[x] > XRIGHT)
		{
			enemy::direction *= -1;

			for(int i = 0; i < MAXSHIPS; i++)
			{
				if (enemies[i]->alive)
					enemies[i]->pos[y] += enemies[i]->maxspeed[y];
			}
		}

		// check for endgame activity
		for(int i = 0; i < MAXSHIPS; i++)
		{
			if (enemies[i]->alive)
			{
				// for all alive enemies: if y coordinate is below 0.5, then they reach the player
				// and the game is over
				if (enemies[i]->pos[y] < 0.5)
					gameOver(false);
			}
		}

		// restrict player firing angle and positions
		if (pplayer.fire_angle > MAXANGLE) pplayer.fire_angle = MAXANGLE;
		if (pplayer.fire_angle < MINANGLE) pplayer.fire_angle = MINANGLE;

		if (pplayer.pos[x] < XLEFT) pplayer.pos[x] = XLEFT;
		if (pplayer.pos[x] > XRIGHT) pplayer.pos[x] = XRIGHT;

		// control player's bullets
		if (pplayer.mybullet != NULL)
		{
			// if the bullet goes out of range, delete it
			if (abs(((bullet*)pplayer.mybullet)->position[x]) > XRIGHT || abs(((bullet*)pplayer.mybullet)->position[y]) > YMAX) 
			{
				pplayer.deleteBullet();
			}

			// otherwise, try to update its position
			if (pplayer.mybullet != NULL) // in case the bullet was deleted
			{
				((bullet*)pplayer.mybullet)->position[x] += ((bullet*)pplayer.mybullet)->speed[x];
				((bullet*)pplayer.mybullet)->position[y] += ((bullet*)pplayer.mybullet)->speed[y];
				((bullet*)pplayer.mybullet)->position[z] += ((bullet*)pplayer.mybullet)->speed[z];
			}

			// check for collision with live enemies
			if (pplayer.mybullet != NULL)
			{
				for(int i = 0; i < MAXSHIPS; i++)
				{
					if (enemies[i]->alive)
					{
						GLfloat distance = computeDistance(enemies[i]->pos, ((bullet*)pplayer.mybullet)->position);
						if (distance < 0.5)
						{
							pplayer.deleteBullet();
							enemies[i]->alive = false;
							activeships--;
							playerscore++;

							if (activeships == 0) gameOver(true);
							break;
						}
					}
				}
			}
		}

		glutPostRedisplay();
	}
}

void keyboard(unsigned char key, int _x, int _y)
{
	// exit on escape
	if (key == 27) exit(0);

	// press a to move player left
	if (key == 97) pplayer.pos[x] -= pplayer.maxspeed[x];
	if (key == 100) pplayer.pos[x] += pplayer.maxspeed[x];

	// press spacebar to fire
	if (key == 32)
	{
		pplayer.fire();
		for(int i = 0; i < MAXSHIPS; i++)
		{
			if (enemies[i]->alive)
			{
				enemies[i]->fire();
			}
		}
	}

	glutPostRedisplay();
};

void special(int key, int _x, int _y)
{
	// use arrow keys to change firing angle
	if (key == GLUT_KEY_LEFT) pplayer.fire_angle += pplayer.angle_speed;
	if (key == GLUT_KEY_RIGHT) pplayer.fire_angle -= pplayer.angle_speed;
}

int main(int args, char **argsv)
{
	// initiate grid of enemies
	for(int i = 0; i < 10; i++)
	{
		for(int j = 0; j < 5; j++)
		{
			enemies[(i * 5) + j] = new enemy(XLEFT + (i * 2.0), 10.0 + j * 2.0, 0.0);
		}
	}

	glutInit(&args, argsv);
	init(); // from wani's lib
	glutDisplayFunc(display); // calls display function from above
	glutIdleFunc(idle);
	glutKeyboardFunc(keyboard);
	glutSpecialFunc(special);
	glutMainLoop();
}